#!/usr/bin/env python2
"""
Update AWS Route53 ALA Dyndns

Usage: dyndns53.py [options]

Options:
  -s SUBDOMAIN --subdomain=SUBDOMAIN    [default: home]
  -d DOMAIN --domain=DOMAIN             [default: example.org]
  -n IP --nameserver=IP                 [default: 8.8.8.8]
  -c CLASSES --class=CLASSES            [default: A,AAAA]
  -t TTL --ttl=TTL                      [default: 900]
  -f --force                            [default: false]
"""


import area53
from area53 import route53
from boto.route53.exception import DNSServerError
from datetime import datetime
from docopt import docopt
import dns.name
import dns.message
import dns.query
import dns.flags
import requests
import socket
import sys


def get_public_ip(rsclass='A'):
    services = {
        'A': [
            'http://curlmyip.com',
            'http://ipinfo.io/ip',
            'http://ifconfig.me',
            'http://icanhazip.com',
            'http://smart-ip.net/myip',
            'http://www.trackip.net/ip'
        ],
        'AAAA': [
            'http://v6.ipv6-test.com/api/myip.php',
            'http://smart-ip.net/myipv6',
        ]
    }

    for service in services[rsclass]:
        r = requests.get(service)
        ip = r.text.rstrip()
        # Validate!
        try:
            if rsclass == 'A':
                socket.inet_pton(socket.AF_INET, ip)
            elif rsclass == 'AAAA':
                socket.inet_pton(socket.AF_INET6, ip)
            return ip
        except socket.error:
            continue
    return None


def dig_direct_nameserver(subdomain, domain, name_server):
    records = dig_ip(domain, name_server)

    for ns in records['NS']:
        try:
            name_server = socket.gethostbyname(ns)
        except socket.gaierror:
            name_server = None
            continue
        if name_server is not None:
            break
        else:
            raise Exception('No nameservers can be looked up!')

    fqdn = '%s.%s' % (subdomain, domain)

    records = dig_ip(fqdn, name_server)
    return records


def dig_ip(domain, name_server):
    ADDITIONAL_RDCLASS = 65535
    domain = dns.name.from_text(domain)
    if not domain.is_absolute():
        domain = domain.concatenate(dns.name.root)

    request = dns.message.make_query(domain, dns.rdatatype.ANY)
    request.flags |= dns.flags.AD
    request.find_rrset(request.additional, dns.name.root, ADDITIONAL_RDCLASS,
                       dns.rdatatype.OPT, create=True, force_unique=True)
    response = dns.query.udp(request, name_server)

    records = {}
    for record in response.answer:
        rdtype = dns.rdatatype.to_text(record.rdtype)
        if rdtype not in records:
            records[rdtype] = []
        records[rdtype].append(str(record.items[0]))

    return records


def main():
    args = docopt(__doc__, version='dyndns53 0.0.1')

    fqdn = '%s.%s' % (args['--subdomain'], args['--domain'])

    records = dig_direct_nameserver(
        args['--subdomain'], args['--domain'], args['--nameserver'])

    for rsclass in args['--class'].split(','):

        new_ip = get_public_ip(rsclass)
        old_ip = records[rsclass][0]

        if new_ip is None:
            print "Failed to lookup public ip for class %s" % (rsclass)
            sys.exit(1)

        datestr = '"Last update %s UTC."'
        datestr = datestr % datetime.utcnow().strftime('%Y-%m-%d %H:%M')

        if old_ip == new_ip and not args['--force']:
            print '%s %s is current. (%s)' % (fqdn, rsclass, new_ip)
            continue

        zone = area53.route53.get_zone(args['--domain'])

        arec = zone.get(resource_type=rsclass, name=fqdn)
        txtrec = zone.get_txt(name=fqdn)

        if arec:
            print 'Updating %s %s: %s -> %s' % (rsclass, fqdn, old_ip, new_ip)

            try:
                zone.update_records(
                    resource_type=rsclass,
                    name=fqdn,
                    new_value=new_ip,
                    new_ttl=args['--ttl'])

            except DNSServerError as e:
                # This can happen if the record did not already exist. Let's
                # try to add_a in case that's the case here.
                print 'Creating %s %s: %s' % (rsclass, fqdn, new_ip)
                zone.add_record(rsclass, fqdn, new_ip, args['--ttl'])
        else:
            print 'Creating %s %s: %s' % (rsclass, fqdn, new_ip)
            zone.add_record(rsclass, fqdn, new_ip, args['--ttl'])

        if txtrec:
            try:
                zone.update_txt(
                    fqdn,
                    datestr,
                    args['--ttl'])
            except DNSServerError as e:
                zone.add_record('TXT', fqdn, datestr, args['--ttl'])
        else:
            zone.add_record('TXT', fqdn, datestr, args['--ttl'])

if __name__ == "__main__":
    main()
